@@ -94,38 +94,38 @@ servrEnv$daemon_list = list()
 # a hint on how to stop the daemonized server
 daemon_hint = function(server) {
   if (!interactive()) return(invisible(server))
-  servrEnv$daemon_list[[length(servrEnv$daemon_list) + 1]] = server
-  message('To stop the server, run servr::daemon_stop("', server, '")',
-          ' or restart your R session')
+  i = length(servrEnv$daemon_list) + 1
+  servrEnv$daemon_list[[i]] = server
+  message('To stop the server, run servr::daemon_stop(', i, ') or restart your R session')
   invisible(server)
 }
 
 #' Utilities for daemonized servers
 #'
-#' The server functions in this package will return server handles. You can pass the
-#' handles to \code{daemon_stop()} to stop the daemonized servers.
-#' @param which the server handles returned by server functions; by default, all
-#'   existing handles in the current R session obtained from
-#'   \code{daemon_list()}, i.e., all daemon servers will be stopped by default
+#' \code{daemon_list()} returns IDs of servers, which can be used to stop the
+#' daemonized servers.
+#' @param which A integer vector of the server IDs; by default, IDs of all
+#'   existing servers in the current R session obtained from
+#'   \code{daemon_list()}, i.e., all daemon servers will be stopped by default.
 #' @return  The function \code{daemon_list()} returns a list of existing server
-#'   handles, and \code{daemon_stop()} returns an invisible \code{NULL}.
+#'   IDs, and \code{daemon_stop()} returns an invisible \code{NULL}.
 #' @export
 daemon_stop = function(which = daemon_list()) {
-  list = daemon_list()
   for (d in which) {
-    i = match(d, list)
-    if (is.na(i)) {
-      warning('The server ', d, ' has been stopped!')
-      next
-    }
-    d = list[[i]]
-    stopServer(d)
-    servrEnv$daemon_list = setdiff(servrEnv$daemon_list, d)
+    if (length(s <- servrEnv$daemon_list[[d]]) == 0) next
+    stopServer(s)
+    servrEnv$daemon_list[[d]] = list()
   }
 }
 #' @rdname daemon_stop
 #' @export
-daemon_list = function() unlist(servrEnv$daemon_list)
+daemon_list = function() {
+  x = seq_along(servrEnv$daemon_list)
+  d = integer()
+  for (i in x) if (length(servrEnv$daemon_list[[i]]) == 0) d = c(d, i)
+  if (length(d)) x = x[-d]
+  if (length(x)) x else invisible(x)
+}
 
 # watch files with pattern, and rebuild them if necessary
 build_watcher = function(pattern, build, dir = getwd()) {
@@ -176,10 +176,19 @@ serve_example = function(name, FUN, ..., run = interactive()) {
   FUN(dir, ...)
 }
 
-# find a random available TCP port (to launch server)
-random_port = function(port = 4321L, host = '127.0.0.1') {
+#' Find a random available TCP port
+#'
+#' Test a series of random TCP ports from 3000 to 8000 (excluding a few that are
+#' considered unsafe by Chrome) and return the first available one. A web server
+#' can be later started on this port.
+#' @param port The preferred port(s).
+#' @param n The maximum number of random ports to be tested.
+#' @inheritParams server_config
+#' @export
+#' @return A port number, or an error if no ports are available.
+random_port = function(port = 4321L, host = '127.0.0.1', n = 20) {
   # exclude ports considered unsafe by Chrome http://superuser.com/a/188070
-  ports = sample(setdiff(3000:8000, c(3659, 4045, 6000, 6665:6669)), 20)
+  ports = sample(setdiff(3000:8000, c(3659, 4045, 6000, 6665:6669)), n)
   ports = c(port, ports)
   port = NULL
   for (p in ports) if (port_available(p, host)) {
@@ -204,7 +213,7 @@ decode_path = function(req) {
   path
 }
 
-paste2 = function(...) paste(c(...), collapse = '\r\n')
+paste2 = function(...) paste(c(...), collapse = '\n')
 
 file_size = function(path) file.info(path)[, 'size']
 
